## configuration

## download *narrowPeaks.gz from
## http://hgdownload.cse.ucsc.edu/goldenPath/hg19/encodeDCC/wgEncodeUwTfbs
## into narrowPeaksDir
narrowPeaksDir <- "~/SequenceAnalysisData/bigdata/chipseq-narrowPeak"
dataPkgDir <- "~/SequenceAnalysisData"

## script

library(GenomicRanges)
library(BSgenome.Hsapiens.UCSC.hg19)

stamFile <- file.path(dataPkgDir, "data", "stam.Rda")
stamMetaFile <- file.path(dataPkgDir, "inst", "extdata",
                          "ENCODE-Stam-metadata.csv")

## 
## meta-data
## 
## file characteristics
url <- "http://hgdownload.cse.ucsc.edu/goldenPath/hg19/encodeDCC/wgEncodeUwTfbs/files.txt"
raw <- readLines(url)
fname <- sapply(strsplit(raw, "\t"), "[[", 1)
fnmeta <- sapply(strsplit(raw, "\t"), "[[", 2)
keyvalue <- strsplit(fnmeta, "; ")
len <- sapply(keyvalue, length)
re0 <- "^(.*)=(.*)$"
re1 <- "wgEncodeUwTfbs(.+)(Ctcf|Input).*Rep([1-3]).*"
cellLine <- rep(sub(re1, "\\1", fname), len)
repl <- rep(sub(re1, "\\3", fname), len)
fmeta <- data.frame(Id=paste(cellLine, repl, sep="_"),
                    CellLine=cellLine, Rep=repl,
                    Key=factor(sub(re0, "\\1", unlist(keyvalue))),
                    Value=sub(re0, "\\2", unlist(keyvalue)),
                    File=rep(fname, len), stringsAsFactors=FALSE)
## peaks file info
peaksId <- with(fmeta, Id[Key == "size" & grepl("narrowPeak.gz", File)])
peaksDate <- as.Date(with(fmeta, {
    Value[Key == "dateSubmitted" & grepl("narrowPeak.gz", File)]
}))
peaksDate <- setNames(peaksDate, peaksId)
## fastq file info
fastqId <- with(fmeta, Id[Key == "size" & grepl("fastq", File)])
fastqDate <- as.Date(with(fmeta, {
    Value[Key == "dateSubmitted" & grepl("fastq", File)]
}))
fastqSize <- with(fmeta, {
    sz0 <- Value[Key == "size" & grepl("fastq", File)]
    scale <- ifelse(grepl("M", sz0), 1, 1000)
    as.numeric(sub("[MG]", "", sz0)) * scale
})
fastqDate <- setNames(fastqDate, fastqId)
fastqSize <- setNames(sz, fastqId)

## narrowPeak.gz files
fls <- list.files(narrowPeaksDir, "narrowPeak.gz$", full=TRUE)
re <- "wgEncodeUwTfbs(.+)CtcfStdPkRep([1-3]).narrowPeak.gz"
peaksFile <- basename(fls)
cellLine <- sub(re, "\\1", peaksFile)
repl <- sub(re, "\\2", peaksFile)
id <- paste(cellLine, repl, sep="_")
meta <- data.frame(CellLine=cellLine, Rep=repl,
                   TotTags=NA_integer_, TotPeaks=NA_integer_,
                   Tags=NA_integer_, Peaks=NA_integer_,
                   FastqDate=fastqDate[id], FastqSize=fastqSize[id],
                   PeaksDate=peaksDate[id], PeaksFile=peaksFile,
                   row.names=id, stringsAsFactors=FALSE)

drop <- with(meta, names(table(CellLine))[table(CellLine) %in% c(1, 3)])
meta <- meta[!meta$CellLine %in% drop,]
meta$Rep <- factor(meta$Rep)
fls <- fls[basename(fls) %in% meta$PeaksFile]

## peaks
read1 <-
    function(fl)
{
    colClasses <- c(seqnames="character", start="integer", end="integer",
                    "NULL", "NULL", "NULL", Tags="integer",
                    P10log10="numeric", "NULL", "NULL")
    csv <- read.csv(fl, header=FALSE, sep="\t", colClasses=colClasses,
                    col.names=names(colClasses))
    csv$seqnames <-
        factor(csv$seqnames, levels=paste("chr", c(1:22, "X", "Y"), sep=""))
    g <- with(csv, GRanges(seqnames, IRanges(start, end),
                           Tags=Tags, P10log10=P10log10))
    seqinfo(g) <- seqinfo(Hsapiens)[seqlevels(g)]
    g
}

## first pass: find common peaks
peaks <- GRanges()
for (fl in fls) {
    message(basename(fl))
    g <- read1(fl)
    values(g) <- NULL
    peaks <- reduce(c(peaks, g))
}

## second pass: tags, P-values
tags <- matrix(0L, length(peaks), length(fls))
pvals <- matrix(0, length(peaks), length(fls))
for (i in seq_along(fls)) {
    message(i)
    fl <- fls[i]
    g <- read1(fl)
    idx <- match(g, peaks)
    meta[i, "TotTags"] <- sum(values(g)[["Tags"]])
    meta[i, "TotPeaks"] <- length(g)
    g <- g[!duplicated(idx)]
    idx <- idx[!duplicated(idx)]
    tags[idx, i] <- values(g)[["Tags"]]
    pvals[idx, i] <- values(g)[["P10log10"]]
}
meta$Peaks <- colSums(tags != 0)
meta$Tags <- colSums(tags)

stam <- SummarizedExperiment(SimpleList(Tags=tags, PVals=pvals),
                             rowData=peaks,
                             colData=as(meta, "DataFrame"))

stamFile <- "~/SequenceAnalysisData/data/stam.Rda"
save(stam, file=stamFile)

write.table(as(colData(stam), "data.frame"), file=stamMetaFile, sep=",",
            col.names=NA)

## fl <- system.file("extdata", "ENCODE-Stam-metadata.csv",
##                   package="SequenceAnalysisData")
## meta <- read.table(fl, header=TRUE, sep=",", row.names=1)
